//=============================================================================
//
// Rubik's Cube Java Applet
//
// Version 1.3 -- May 2 1999
//
// Copyright 2000 Michael Schubart (michael at schubart dot net)
// http://www.schubart.net/
//
// The applet plus additional information can be found at
// http://www.schubart.net/rc/
//
//=============================================================================

package net.schubart.rubiks;

import java.applet.Applet;
import java.applet.AudioClip;
import java.awt.BorderLayout;
import java.awt.Button;
import java.awt.Checkbox;
import java.awt.Event;
import java.awt.Frame;
import java.awt.Graphics;
import java.awt.Image;
import java.awt.image.CropImageFilter;
import java.awt.image.FilteredImageSource;
import java.awt.image.ImageFilter;
import java.awt.Label;
import java.awt.MediaTracker;
import java.awt.Panel;
import java.awt.TextField;

//=============================================================================
// The RubiksCubeApplet class implements a simulation of Rubik's Cube.
//=============================================================================

public class RubiksCubeApplet extends Applet {

    // names for the shapes (the different looks cubelet faces can have)
    public final static int SHAPE_XF = 0; // on FACE_X, fully visible

    public final static int SHAPE_XP = 1; // on FACE_X, partialy visible

    public final static int SHAPE_YF = 2; // and so on

    public final static int SHAPE_YP = 3;

    public final static int SHAPE_ZF = 4;

    public final static int SHAPE_ZP = 5;

    // can be used to translate the graphics within the applet
    public final static int CUBEXOFFSET = 30;

    public final static int CUBEYOFFSET = 0;

    // describes where and how each face gets drawn
    // 6 cube faces, each with 3*3 cubelet faces -> 52 visible cubelet faces
    public final static RCDisplayEntry[] displayTable = new RCDisplayEntry[54];

    //=========================================================================
    // This unnamed static function initializes the display table. Don't ask me
    // to explain the computations. It was a dirty job, but someone had to do
    // it...
    //=========================================================================
    static {
        int i = 0;

        for (int y = 0; y < 3; y++) {
            for (int z = 0; z < 3; z++) {
                displayTable[i++] = new RCDisplayEntry(2, y, z,
                        RCCubelet.FACE_XV, 211 - 28 * z, 153 - 32 * y + 14 * z,
                        SHAPE_XF);

                displayTable[i++] = new RCDisplayEntry(0, y, z,
                        RCCubelet.FACE_XH, 58 - 28 * z, 67 - 32 * y + 14 * z,
                        (y == 0 && z == 0) ? SHAPE_XP : SHAPE_XF);
            }
        }

        for (int x = 0; x < 3; x++) {
            for (int z = 0; z < 3; z++) {
                displayTable[i++] = new RCDisplayEntry(x, 2, z,
                        RCCubelet.FACE_YV, 129 + 28 * x - 28 * z, 46 + 14 * x
                                + 14 * z, SHAPE_YF);

                displayTable[i++] = new RCDisplayEntry(x, 0, z,
                        RCCubelet.FACE_YH, 129 + 28 * x - 28 * z, 210 + 14 * x
                                + 14 * z, (x == 0 && z == 0) ? SHAPE_YP
                                : SHAPE_YF);
            }
        }

        for (int x = 0; x < 3; x++) {
            for (int y = 0; y < 3; y++) {
                displayTable[i++] = new RCDisplayEntry(x, y, 2,
                        RCCubelet.FACE_ZV, 71 + 28 * x, 153 - 32 * y + 14 * x,
                        SHAPE_ZF);

                displayTable[i++] = new RCDisplayEntry(x, y, 0,
                        RCCubelet.FACE_ZH, 224 + 28 * x, 67 - 32 * y + 14 * x,
                        (x == 0 && y == 0) ? SHAPE_ZP : SHAPE_ZF);
            }
        }
    }

    // the background image
    protected static Image background;

    // images of the shapes (6 shapes in 6 colors)
    protected static Image[][] shape = new Image[6][6];

    // audio clips
    protected static AudioClip moveSound;

    protected static AudioClip badSound;

    // the cube itself
    protected RCCube cube = new RCCube();

    // where did the user start dragging with the mouse?
    protected RCSurfacePoint dragStart;

    // did they press the Ctrl key then?
    protected boolean dragCtrlKey;

    // do we count the number of moves?
    protected boolean countMoves = false;

    // the components
    protected Button scramble = new Button("Scramble");

    protected Button giveUp = new Button("Give Up");

    protected TextField moves = new TextField(5);

    protected Checkbox sound = new Checkbox("Sound");

    protected Frame success = new RCWinFrame();

    public void init() {
        Panel p = new Panel();
        Panel b = new Panel();
        Panel o = new Panel();

        // create a RCSurfacePoint object - makes sure the class has been
        // loaded when the user starts playing
        new RCSurfacePoint(0, 0, 0, 0);

        // we want something on the bottom
        setLayout(new BorderLayout());
        add("South", p);

        // on the left side: 2 buttons, 1 checkbox
        p.setLayout(new BorderLayout());
        p.add("West", b);
        b.add(scramble);
        b.add(giveUp);
        b.add(sound);
        sound.setState(true);

        // on the right side: label and text field
        p.add("East", o);
        o.add(new Label("Moves:"));
        o.add(moves);
        // text field is read only
        moves.setEditable(false);

        // have the images been loaded by another instance?
        if (background == null) {
            MediaTracker tracker = new MediaTracker(this);
            Image shapes;

            showStatus("Loading images, please wait...");
            background = getImage(getClass().getResource("resources/back.gif"));
            tracker.addImage(background, 0);
            shapes = getImage(getClass().getResource("resources/shapes.gif"));

            for (int colorNo = 0; colorNo < 6; colorNo++) {
                for (int shapeNo = 0; shapeNo < 6; shapeNo++) {
                    // for each shape/color combination: crop a small image
                    // out of the image with all shapes. avoids loading
                    // 54 GIFs through the net.
                    ImageFilter crop = new CropImageFilter(colorNo * 50,
                            shapeNo * 42, 50, 42);
                    shape[colorNo][shapeNo] = createImage(new FilteredImageSource(
                            shapes.getSource(), crop));
                    tracker.addImage(shape[colorNo][shapeNo], 0);
                }
            }

            // wait for all images
            try {
                tracker.waitForAll();
            } catch (InterruptedException e) {
            }
        }

        // load sounds
        moveSound = getAudioClip(getClass().getResource("resources/move.au"));
        badSound = getAudioClip(getClass().getResource("resources/bad.au"));
    }

    //=========================================================================
    // Gets called when the user selects Applet|Info in appletviewer.
    // return value: author, verison and copyright info
    //=========================================================================
    public String getAppletInfo() {
        return "RubiksCubeApplet\n" + "Version 1.3 -- May 2 1999\n\n"
                + "Copyright 2000 Michael Schubart\n"
                + "(michael at schubart dot net)\n"
                + "http://www.schubart.net/";
    }

    //=========================================================================
    // Gets called when the user selects Applet|Info in appletviewer.
    // return value: what parameters does the applet understand?
    //=========================================================================
    public String[][] getParameterInfo() {
        String[][] pinfo = { { "", "No parameters.", ""},};

        return pinfo;
    }

    //=========================================================================
    // Gets called when the applet needs to be redrawn.
    //=========================================================================
    public void paint(Graphics g) {
        // workaround for bug in some versions of Java (like 1.0.2 on Win32)
        g = getGraphics();

        // first the background...
        g.drawImage(background, CUBEXOFFSET, CUBEYOFFSET, this);

        // ...then the cubelet faces
        for (int faceNo = 0; faceNo < displayTable.length; faceNo++) {
            drawFace(g, faceNo);
        }
    }

    //=========================================================================
    // Draws one face of one cubelet.
    // faceNo: the number of the display table entry that describes which
    //          cubelet face gets drawn where
    //=========================================================================
    public void drawFace(Graphics g, int faceNo) {
        // get the shape
        int shapeNo = displayTable[faceNo].shape;

        // get the color
        int colorNo = cube.getColor(displayTable[faceNo].xSource,
                displayTable[faceNo].ySource, displayTable[faceNo].zSource,
                displayTable[faceNo].faceSource);

        // then draw it
        g
                .drawImage(shape[colorNo][shapeNo],
                        displayTable[faceNo].xOffset + CUBEXOFFSET,
                        displayTable[faceNo].yOffset + CUBEYOFFSET, this);
    }

    //=========================================================================
    // Gets called when the user clicks with the mouse. We start observing
    // what the mouse does.
    //=========================================================================
    public boolean mouseDown(Event event, int x, int y) {
        // are we for some reason already observing the mouse?
        if (dragStart != null) { 
        // then ignore the click
        return true; }

        // where did the user click?
        dragStart = new RCSurfacePoint(x, y, CUBEXOFFSET, CUBEYOFFSET);
        // was the Ctrl key pressed?
        dragCtrlKey = (event.modifiers & java.awt.Event.CTRL_MASK) != 0;

        // is it a point on the cube?
        if (!dragStart.isOnCube()) {
            // no -> forget that point, don't observe the mouse
            dragStart = null;
            if (sound.getState()) {
                badSound.play();
            }
        }

        return true;
    }

    //=========================================================================
    // Gets called when the user releases the mouse button. We compute a
    // vector from the 2 points where the mouse button was clicked and
    // released. If it's a legal command, we twist the cube accordingly.
    //=========================================================================
    public boolean mouseUp(Event event, int x, int y) {
        // are we observing the mouse?
        if (dragStart == null) { 
        // no -> ignore the event
        return true; }

        // what colors are displayed where?
        int[] cache = new int[displayTable.length];
        RCSurfacePoint dragStop = new RCSurfacePoint(x, y, CUBEXOFFSET,
                CUBEYOFFSET, dragStart.face);

        // how are the 2 points relative to each other?
        int deltaX = mysign(dragStop.cubeX - dragStart.cubeX);
        int deltaY = mysign(dragStop.cubeY - dragStart.cubeY);
        int deltaZ = mysign(dragStop.cubeZ - dragStart.cubeZ);

        // did we twist the cube?
        boolean rotation = false;

        // get all the old color values
        for (int faceNo = 0; faceNo < displayTable.length; faceNo++) {
            cache[faceNo] = cube.getColor(displayTable[faceNo].xSource,
                    displayTable[faceNo].ySource, displayTable[faceNo].zSource,
                    displayTable[faceNo].faceSource);
        }

        // twist the cube according to the 2 points
        switch (dragStart.face) {
        case RCCubelet.FACE_XV:
            if (deltaY == 0 && deltaZ != 0) {
                cube.rotate(RCCubelet.AXIS_Y, -deltaZ, dragStart.cubeY,
                        dragCtrlKey);
                rotation = true;

            } else if (deltaY != 0 && deltaZ == 0) {
                cube.rotate(RCCubelet.AXIS_Z, deltaY, dragStart.cubeZ,
                        dragCtrlKey);
                rotation = true;
            }
            break;

        case RCCubelet.FACE_YV:
            if (deltaX == 0 && deltaZ != 0) {
                cube.rotate(RCCubelet.AXIS_X, deltaZ, dragStart.cubeX,
                        dragCtrlKey);
                rotation = true;

            } else if (deltaX != 0 && deltaZ == 0) {
                cube.rotate(RCCubelet.AXIS_Z, -deltaX, dragStart.cubeZ,
                        dragCtrlKey);
                rotation = true;
            }
            break;

        case RCCubelet.FACE_ZV:
            if (deltaX == 0 && deltaY != 0) {
                cube.rotate(RCCubelet.AXIS_X, -deltaY, dragStart.cubeX,
                        dragCtrlKey);
                rotation = true;

            } else if (deltaX != 0 && deltaY == 0) {
                cube.rotate(RCCubelet.AXIS_Y, deltaX, dragStart.cubeY,
                        dragCtrlKey);
                rotation = true;
            }
            break;
        }

        // stop observing the mouse
        dragStart = null;

        // did we twist it?
        if (rotation) {
            Graphics g = getGraphics();

            // success message stil visible? -> hide
            if (success.isVisible()) {
                success.hide();
            }

            // make some noise
            if (sound.getState() && !dragCtrlKey) {
                moveSound.play();
            }

            // draw only those faces that have a different color now
            for (int faceNo = 0; faceNo < displayTable.length; faceNo++) {
                int cacheColor = cache[faceNo];
                int color = cube.getColor(displayTable[faceNo].xSource,
                        displayTable[faceNo].ySource,
                        displayTable[faceNo].zSource,
                        displayTable[faceNo].faceSource);

                if (color != cacheColor) {
                    drawFace(g, faceNo);
                }
            }

            // do we currently count the moves and has the cube's
            // cube state really changed?
            if (countMoves && !dragCtrlKey) {
                int count = Integer.parseInt(moves.getText());

                // increase the number of moves displayed by 1
                moves.setText(String.valueOf(count + 1));

                // are we there yet?
                if (cube.isSolved()) {
                    // stop counting
                    countMoves = false;
                    showStatus("You win!");

                    // show message
                    success.show();
                }
            }
        } else if (sound.getState()) {
            badSound.play();
        }

        return true;
    }

    //=========================================================================
    // Convenience function.
    // return value: 1 if val > 0
    //                 0 if val = 0
    //                -1 if val < 0
    //=========================================================================
    protected int mysign(int val) {
        return (val == 0) ? 0 : ((val > 0) ? 1 : -1);
    }

    //=========================================================================
    // Gets called when the user pressed one of the buttons.
    //=========================================================================
    public boolean action(Event event, Object what) {
        // was it the "Scramble" button?
        if (event.target == scramble) {
            // start counting moves
            countMoves = true;

            // initialize moves counter
            moves.setText("0");

            // start with a clean cube, forget the old one, let garbage
            // collection take care of it
            cube = new RCCube();

            // scamble it
            showStatus("Scramble, scramble...");
            cube.scramble(500);

            // see what you've done!
            repaint();
            return true;
        }

        // was it the "Give Up" button?
        if (event.target == giveUp) {
            // stop counting moves
            countMoves = false;

            // clear counter
            moves.setText("");

            // create a clean cube, dump the old one
            cube = new RCCube();

            // show it
            repaint();
            return true;
        }

        return false;
    }
}
